#ifndef SHIP_H_
#define SHIP_H_

#include "Optimal_Planner.hpp"
#include "USV.hpp"
#include "Guidance.hpp"
#include "Motion_Prediction.hpp"
#include "COLREGs.hpp"

class Ship
{
public:

    int ship_idx;
    std::vector<Eigen::Vector2d> wps, map;
    TebConfig cfg;
    USV usv;
    Guidance guidance;

    Eigen::VectorXd eta_6;
    PoseSE2 eta_3;

    Eigen::VectorXd nu_6;
    Eigen::Vector3d nu_3;

    Eigen::Vector2d n, n_c;

    double psi_ref, psi_d, r_d, alpha_d;
    double u_nominal, u_ref, u_d, a_d;

    std::vector<double>
    data_x, data_y,
    data_psi_ref, data_psi_d, data_psi,
    data_u_ref, data_u_d, data_u,
    data_x_, data_y_, data_psi_;

    std::vector<std::pair<PoseSE2,Eigen::Vector3d>> motion_data;

    bool safe;
    double time;
    std::vector<std::string> encounter_types;
    RotType prefer_rotdir;

    Ship(ShipConfig& ship_cfg);
    void recordData();
    void recordPose();
    std::pair<PoseSE2,Eigen::Vector3d> provideMotionData();
    std::string getMode();
    void collisionAvoidance();
    void pathFollowing();
    void simulate(double& current_time, std::vector<std::pair<PoseSE2,Eigen::Vector3d>>& ship_motion_data);
    void simulate(double& current_time);

public:
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW
};

#endif