#ifndef EDGE_GOAL_H_
#define EDGE_GOAL_H_

#include "Vertex_Pose.hpp"
#include "Edge_TEB_Base.hpp"
#include "../TEB_Config.hpp"


namespace teb_local_planner
{
 
class EdgeGoal : public BaseTebUnaryEdge<1, const PoseSE2*, VertexPose>
{
public:
    
  /**
   * @brief Construct edge.
   */    
  EdgeGoal() 
  {
    _measurement = NULL;
  }
 
  /**
   * @brief Actual cost function
   */    
  void computeError()
  {
    const VertexPose* bandpt = static_cast<const VertexPose*>(_vertices[0]);
    double dist = (bandpt->position() - _measurement->position()).norm();

      _error[0] = dist;
  }

  /**
   * @brief Set all parameters at once
   * @param cfg TebConfig class
   * @param robot_model Robot model required for distance calculation
   * @param obstacle 2D position vector containing the position of the obstacle
   */ 
  void setParameter(const TebConfig& cfg, const PoseSE2& goal)
  {
    cfg_ = &cfg;
    _measurement = &goal;
  }
  
public: 	
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW

};

}

#endif