#ifndef MISC_H_
#define MISC_H_

#include <Eigen/Core>
#include <Eigen/Dense>
#include "Pose_SE2.hpp"

using namespace teb_local_planner;

struct ShipConfig
{
    int index;
    PoseSE2 eta0;
    Eigen::Vector3d nu0;
    double u_nominal;
    std::vector<Eigen::Vector2d> waypoints;
    std::vector<Eigen::Vector2d> map;
};

struct Ellipse
{
    Eigen::Vector2d position;
    double a, b, start, end, angle = 0;
};

double D2R(double x);
double R2D(double x);

Eigen::Matrix3d Smtrx(Eigen::Vector3d a);

Eigen::MatrixXd Hmtrx(Eigen::Vector3d r);

Eigen::MatrixXd m2c(Eigen::MatrixXd& M, Eigen::VectorXd& nu);

double sat(double x, double x_min, double x_max);

Eigen::VectorXd crossFlowDrag(double L, double B, double T, Eigen::VectorXd nu_r);

Eigen::Matrix3d Rzyx(double phi, double theta, double psi);

Eigen::Matrix3d Tzyx(double phi, double theta);

Eigen::Matrix3d R_psi(double psi);


void record(double data, std::vector<double>& container);

bool inInterval(double x, double lowerbound, double upperbound);

int fact(int n);

int combination(int n);

void update_indices(int& idx1, int& idx2, int idx_max);

std::vector<std::string> get_ASV_ASV(int n);

std::pair<double,int> getMinDistance(std::vector<Eigen::Vector2d> obstacles, Eigen::Vector2d position);

std::vector<Eigen::Vector2d> getNearbyObstacles(Eigen::Vector2d& position, std::vector<Eigen::Vector2d>& static_obstacles, std::vector<Eigen::Vector2d>& dynamic_obstacles, double& range);

std::vector<Eigen::Vector2d> getEllipticObstacle(Eigen::Vector2d& position, double& a, double& b, double& start, double& end, double& angle);

#endif