#include "../include/Plot.hpp"

container2 packData(data& data_x_, data& data_y_, data& data_psi_, data& data_x, data& data_y)
{
    return {container1{data_x_, data_y_, data_psi_}, container1{data_x, data_y}};
}

container2 packData(data& data_psi_ref, data& data_psi_d, data& data_psi, data& data_u_ref, data& data_u_d, data& data_u)
{
    return {container1{data_psi_ref, data_psi_d, data_psi}, container1{data_u_ref, data_u_d, data_u}};
}

std::pair<data,data> get_shape(double scale, double x0, double y0, double psi)
{
    double length = 2;
    double width = 1.08;
    double bow = 3*width/4;

    double L = scale * length/2;
    double B = scale * width/2;
    double T = scale * bow;
    double l = sqrt(pow(B,2)+pow(L,2));
    double beta = atan(B/L);

    data x = { x0+(L+T)*cos(psi), x0+l*cos(psi+beta), x0-l*cos(psi-beta), x0-l*cos(psi+beta), x0+l*cos(psi-beta) };
    data y = { y0+(L+T)*sin(psi), y0+l*sin(psi+beta), y0-l*sin(psi-beta), y0-l*sin(psi+beta), y0+l*sin(psi-beta) };

    return {x,y};
}

void plotXY(container3& ship_data, int scenario)
{
    auto xy = figure();
    xy->x_position(10);
    xy->y_position(10);
    xy->name("Scenario "+std::to_string(scenario)+" xy plot");
    xy->size(600, 600);
    xlim({-100,100});
    ylim({-100,100});
    xticks({-100, -80, -60, -40, -20, 0, 20, 40, 60, 80, 100});
    yticks({-100, -80, -60, -40, -20, 0, 20, 40, 60, 80, 100});
    xlabel("East (m)");
    ylabel("North (m)");
    grid(on);
    axis(equal);
    double scale = 1.75;
    std::vector<std::array<float,3>>
    color_asv = {{0.40,0.67,0.93},{1,0.40,0.34},{0.42,0.85,0.38},{1,0.71,0.31},{0.43,0.97,1}},
    color_path = {{0.12,0.47,0.71},{0.84,0.15,0.16},{0.17,0.63,0.17},{1,0.5,0.05},{0.09,0.75,0.81}};
    std::vector<std::string> ASVs;

    for (int i = 0; i < ship_data.size(); ++i)
        ASVs.push_back("ASV "+ std::to_string(i+1));

    for (int i = 0; i < ship_data.size(); ++i)
    {
        auto path = plot(ship_data[i].second[1], ship_data[i].second[0]);
        path->color(color_path[i]);
        path->line_width(1.2);
        hold(on);
    }

    // for (int i = 0; i < ship_data[0].first[0].size(); ++i)
    // {
    //     for (int j = 0; j < ship_data.size(); ++j)
    //     {
    //         double x0 = ship_data[j].first[0][i];
    //         double y0 = ship_data[j].first[1][i];
    //         double psi = ship_data[j].first[2][i];

    //         auto [x,y] = get_shape(scale,x0,y0,psi);
    //         auto ASV_shape = fill(y,x);
    //         ASV_shape->color(color_asv[j]);
    //         ASV_shape->line_width(0.1);
    //         hold(on);
    //         x.push_back(x.front());
    //         y.push_back(y.front());
    //         auto ASV_outline = plot(y,x);
    //         ASV_outline->line_width(1.1);
    //         ASV_outline->color({0,0,0});
    //         hold(on);
    //     }
    // }

    auto lgd = legend(ASVs);
    lgd->box(off);
    lgd->font_size(8);
}

void plotDynamics(data& time, container3& ship_data, int scenario, bool limit_time)
{
    for (int i = 0; i < ship_data.size(); ++i)
    {
        data psi_ref = ship_data[i].first[0];
        data psi_d = ship_data[i].first[1];
        data psi = ship_data[i].first[2];
        data u_ref = ship_data[i].second[0];
        data u_d = ship_data[i].second[1];
        data u = ship_data[i].second[2];
        // yaw
        auto yaw = figure(false);
        yaw->name("Scenario "+std::to_string(scenario)+" ASV "+ std::to_string(i+1) + " yaw angle");
        yaw->size(500,375);
        xlabel("time (s)");
        ylabel("\u03C8 ( °)");
        if (limit_time)
        {
            if (time.back() > 140)
                xlim({0,140});
        }
        ylim({-180,180});
        yticks({-180, -135, -90, -45, 0, 45, 90, 135, 180});
        // plot(time, psi_ref)->line_width(1.2);
        // hold(on);
        plot(time, psi_d)->line_width(1.2);
        hold(on);
        plot(time, psi,"--")->line_width(1.2).color("red");
        // auto lgd_yaw = legend("Reference","Desired", "Actual");
        auto lgd_yaw = legend("Desired", "Actual");
        lgd_yaw->box(false);
        lgd_yaw->font_size(8);
        grid(on);
        
        // speed
        auto speed = figure(false);
        speed->name("case"+std::to_string(scenario)+"ASV "+ std::to_string(i+1)+" surge speed");
        speed->size(500,375);
        xlabel("time (s)");
        ylabel("u (m/s)");
        if (limit_time)
        {
            if (time.back() > 140)
                xlim({0,140});
        }
        ylim({0,3.5});
        yticks({0,0.5,1,1.5,2,2.5,3,3.5});
        // plot(time, u_ref)->line_width(1.2);
        // hold(on);
        plot(time, u_d)->line_width(1.2);
        hold(on);
        plot(time, u,"--")->line_width(1.2).color("red");
        // auto lgd_surge = legend("Reference","Desired", "Actual");
        auto lgd_surge = legend("Desired", "Actual");
        lgd_surge->box(false);
        lgd_surge->font_size(8);
        grid(on);
    }
}

void plotDistance(data& time, container1& dist, std::vector<std::string> ASV_ASV, int scenario, int n_ASV, bool limit_time)
{   
    auto distance = figure();
    distance->x_position(15);
    distance->y_position(15);
    distance->name("Scenario "+std::to_string(scenario)+" distance");
    distance->size(500,375);
    grid(on);
    xlabel("time (s)");
    ylabel("d (m)");
    if (limit_time)
    {
        if (time.back() > 140)
            xlim({0,140});
    }
    ylim({0,200});
    yticks({0, 25, 50, 75, 100, 125, 150, 175, 200});
    std::vector<std::array<float,3>>
    colors1 = {{0.12,0.47,0.71},{1,0.5,0.05},{0.17,0.63,0.17},{0.84,0.15,0.16},{0.58,0.40,0.74},{0.55,0.34,0.29},{0.89,0.47,0.76},{0.5,0.5,0.5},{0.74,0.74,0.13},{0.09,0.75,0.81}},
    colors2 = {{0.71,0.36,0.12},{0.05,0.56,1},{0.63,0.17,0.63},{0.15,0.84,0.84},{0.56,0.74,0.4},{0.29,0.51,0.55},{0.47,0.89,0.60},{0.31,0.31,0.31},{0.14,0.13,0.74},{0.81,0.16,0.09}};

    auto lgd = legend();
    lgd->box(off);
    lgd->font_size(8);
    lgd->horizontal(on);
    lgd->location(legend::general_alignment::top);
    lgd->num_columns(2);

    for (int i=0; i < dist.size(); ++i)
    {
        plot(time, dist[i])->line_width(1.2).color(colors1[i]).display_name(ASV_ASV[i]);
        hold(on);
    }

    int x = 1, y = 2;
    for (int i=0; i < dist.size(); ++i)
    {   
        std::string asv_asv = std::to_string(x)+"-"+std::to_string(y);
        auto it = std::min_element(dist[i].begin(),dist[i].end());
        auto idx = std::distance(dist[i].begin(), it);
        double dist_min = *it;
        std::cout << asv_asv + " minimum distance = " << dist_min << std::endl;
        plot(data{time[idx]} ,data{dist[i][idx]},"*")
        ->color(colors2[i])
        .marker_color(colors2[i])
        .marker_size(6)
        .display_name("d_{min, " +asv_asv+ "} = "+std::to_string(dist_min)
        .substr(0, 4));
        hold(on);
        update_indices(x,y,n_ASV);
    }
}

void drawEllipse(Eigen::Vector2d position, double a, double b, double angle)
{
    double c = sqrt(pow(a,2)+pow(b,2));
    data theta = iota(D2R(-180), D2R(1), D2R(180));
    data x,y;

    for (int i=0; i<theta.size(); ++i)
    {
        double dx = a*cos(theta[i]);
        double dy = b*sin(theta[i]);
        double dx_rotated = dx*cos(D2R(angle)) - dy*sin(D2R(angle));
        double dy_rotated = dx*sin(D2R(angle)) + dy*cos(D2R(angle));
        x.push_back(position(0)+dx_rotated);
        y.push_back(position(1)+dy_rotated);
    }

    auto outline = plot(y,x);
    outline->color("k");
    outline->line_width(2.5);
    auto ellipse = fill(y,x);
    ellipse->color({0.85,0.85,0.85});
    ellipse->line_width(1);
}

void plotXY(container3& ship_data, std::vector<Ellipse> map_details, std::vector<std::vector<Eigen::Vector2d>> waypoints)
{
    auto xy = figure(false);
    xy->x_position(10);
    xy->y_position(10);
    xy->name("xy plot");
    xy->size(600, 600);
    xlim({-100,100});
    ylim({-100,100});
    xticks({-100, -80, -60, -40, -20, 0, 20, 40, 60, 80, 100});
    yticks({-100, -80, -60, -40, -20, 0, 20, 40, 60, 80, 100});
    xlabel("East (m)");
    ylabel("North (m)");
    grid(on);
    axis(equal);

    double scale = 1.75;
    std::vector<std::string> ASVs;
    std::vector<std::array<float,3>>
    color_asv = {{0.40,0.67,0.93},{1,0.40,0.34},{0.42,0.85,0.38},{1,0.71,0.31},{0.43,0.97,1}},
    color_path = {{0.12,0.47,0.71},{0.84,0.15,0.16},{0.17,0.63,0.17},{1,0.5,0.05},{0.09,0.75,0.81}},
    color_wps_asv1 = {{0.95,0.56,0.32},{0.86,0.42,0.22},{0.71,0.36,0.21}},
    color_wps_asv2 = {{0.47,1,1},{0.33,0.95,0.95},{0.15,0.84,0.84}},
    color_wps_asv3 = {{0.85,0.40,0.84},{0.74,0.29,0.73},{0.63,0.17,0.63}};
    std::vector<std::vector<std::array<float,3>>> color_wps;
    color_wps.push_back(color_wps_asv1);
    color_wps.push_back(color_wps_asv2);
    color_wps.push_back(color_wps_asv3);
    
    for (int i = 0; i < ship_data.size(); ++i)
    {
        ASVs.push_back("ASV "+ std::to_string(i+1));
        auto path = plot(ship_data[i].second[1], ship_data[i].second[0]);
        path->color(color_path[i]);
        path->line_width(1.2);
        hold(on);
    }
    
    for(int i = 0; i < waypoints.size(); ++i)
    {
        for (int j = 1; j < waypoints[i].size(); ++j)
        {
            auto WP = plot(data{waypoints[i][j](1)}, data{waypoints[i][j](0)},"x");
            WP->line_width(5);
            WP->color(color_wps[i][2]);
            WP->marker_color(color_wps[i][2]);
            WP->marker_size(15);
            WP->marker_face(true);
        }
    }

    for (int i = 0; i < ship_data[0].first[0].size(); ++i)
    {
        for (int j = 0; j < ship_data.size(); ++j)
        {
            double x0 = ship_data[j].first[0][i];
            double y0 = ship_data[j].first[1][i];
            double psi = ship_data[j].first[2][i];

            auto [x,y] = get_shape(scale,x0,y0,psi);
            auto ASV_shape = fill(y,x);
            ASV_shape->color(color_asv[j]);
            ASV_shape->line_width(0.1);
            hold(on);
            x.push_back(x.front());
            y.push_back(y.front());
            auto ASV_outline = plot(y,x);
            ASV_outline->line_width(1.1);
            ASV_outline->color({0,0,0});
            hold(on);
        }
    }

    for (int i=0; i<map_details.size(); ++i)
    {
        auto e = map_details[i];
        drawEllipse(e.position, e.a, e.b, e.angle);
        hold(on);
    }

    for(int i = 0; i < waypoints.size(); ++i)
    {
        for (int j = 1; j < waypoints[i].size(); ++j)
        {
            auto WP_path = plot(data{waypoints[i][j-1](1),waypoints[i][j](1)},
                           data{waypoints[i][j-1](0),waypoints[i][j](0)},"--");
            WP_path->line_width(1.2);
            WP_path->color(color_path[i]);
            hold(on);
        }
    }

    auto lgd = legend(ASVs);
    lgd->box(off);
    lgd->font_size(8);
    lgd->location(legend::general_alignment::topleft);
}