#include "../include/TEB_Config.hpp"

namespace teb_local_planner
{

TebConfig::TebConfig()
{
    // Trajectory
    trajectory.teb_autosize = true;
    trajectory.dt_ref = 2;
    trajectory.dt_hysteresis = 0.3*trajectory.dt_ref;
    trajectory.min_samples = 3;
    trajectory.max_samples = 500;
    trajectory.exact_arc_length = true;
    trajectory.force_reinit_new_goal_dist = 0;
    trajectory.force_reinit_new_goal_angular = 0;
    trajectory.distance_step = 2.5*trajectory.dt_ref;

    // ship
    robot.max_vel_x = 2.5;
    robot.min_vel_x = 0;
    robot.acc_lim_x = robot.max_vel_x / 10;
    robot.max_vel_theta = 10 * M_PI / 180;
    robot.acc_lim_theta = robot.max_vel_theta / 10;
    robot.min_turning_radius = 3;

    // Obstacles
    obstacles.min_obstacle_dist = 16;
    obstacles.num = 4;

    // Optimization
    optim.no_inner_iterations = 10;
    optim.no_outer_iterations = 1;
    optim.optimization_activate = true;
    optim.optimization_verbose = false;
    optim.weight_colregs = 1e6;
    optim.weight_acc_lim_x = 1e4;
    optim.weight_acc_lim_theta = 1e4;
    optim.weight_max_vel_x = 1e4;
    optim.weight_max_vel_theta = 1e4;
    optim.weight_kinematics_nh = 1e4;
    optim.weight_kinematics_forward_drive = 1e4;
    optim.weight_obstacle = 100;
    optim.weight_min_turning_radius = 100;
    optim.weight_optimaltime = 0;
    optim.weight_goal = 1;
    optim.penalty_epsilon = 0;

    // Planning
    planning.execution_time = 1;
}

}  // namespace teb_local_planner
