#include "../include/Optimal_Planner.hpp"
#include <matplot/matplot.h>

using namespace teb_local_planner;
using namespace matplot;
using namespace std;

int main()
{   
    TebConfig config;
    TebOptimalPlanner localPlanner(config);

    PoseSE2 eta(0,0,0), eta_goal(100,1,0);
    Eigen::Vector3d nu(0, 0, 0), nu_goal(2,0,0);
    std::vector<Eigen::Vector2d> waypoints, obstacles;
    obstacles.push_back(Eigen::Vector2d(50,0));

    localPlanner.plan(eta, eta_goal, nu, nu_goal, obstacles, RotType::none);
    Trajectory trajectory = localPlanner.getTrajectory();
    std::vector<Eigen::Vector2d> command = localPlanner.getCommand(trajectory);

    std::vector<double> time_vec, north_vec, east_vec, theta_vec, u_vec, r_vec;

    for (int i=0; i < trajectory.size(); ++i)
    {
        double current_time = get<0>(trajectory[i]);
        PoseSE2 Pose = get<1>(trajectory[i]);
        double North = get<1>(trajectory[i]).x();
        double East = get<1>(trajectory[i]).y();
        double Heading = get<1>(trajectory[i]).theta() * 180/M_PI; //deg
        double SurgeSpeed = get<2>(trajectory[i])(0);
        double YawRate = get<2>(trajectory[i])(2) * 180/M_PI; // deg/s

        time_vec.push_back(current_time);
        north_vec.push_back(North);
        east_vec.push_back(East);
        theta_vec.push_back(Heading);
        u_vec.push_back(SurgeSpeed);
        r_vec.push_back(YawRate);

        cout << "time: " << current_time << "\n";
        cout << "pose: " << Pose << "\n";
        cout << "speed: " << SurgeSpeed <<"\n" << "yaw rate: " << YawRate << "\n" << endl;
    }

    for (int i=0; i < command.size(); ++i)
    {
        cout << "heading: " << command[i](0) << "\n";
        cout << "velocity: " << command[i](1) << "\n";
    }

    figure(1);
    plot(east_vec, north_vec);
    hold(on);
    plot(east_vec, north_vec, "o");
    show();

    figure(2);
    subplot(2,1,0);
    plot(time_vec, theta_vec);
    hold(on);
    plot(time_vec, theta_vec, "o");
    subplot(2,1,1);
    plot(time_vec, r_vec);
    hold(on);
    plot(time_vec, r_vec, "o");
    show();

    figure(3);
    plot(time_vec, u_vec);
    hold(on);
    plot(time_vec, u_vec, "o");
    show();
}